@@ -5,6 +5,7 @@
 #include "store-api.hh"
 #include "legacy.hh"
 #include "fetchers.hh"
+#include "eval-settings.hh" // for defexpr
 #include "util.hh"
 
 #include <fcntl.h>
@@ -165,7 +166,7 @@ static int main_nix_channel(int argc, char ** argv)
         // Figure out the name of the `.nix-channels' file to use
         auto home = getHome();
         channelsList = settings.useXDGBaseDirectories ? createNixStateDir() + "/channels" : home + "/.nix-channels";
-        nixDefExpr = settings.useXDGBaseDirectories ? createNixStateDir() + "/defexpr" : home + "/.nix-defexpr";
+        nixDefExpr = getNixDefExpr();
 
         // Figure out the name of the channels profile.
         profile = profilesDir() +  "/channels";
@@ -177,6 +178,7 @@ static int main_nix_channel(int argc, char ** argv)
             cRemove,
             cList,
             cUpdate,
+            cListGenerations,
             cRollback
         } cmd = cNone;
         std::vector<std::string> args;
@@ -193,6 +195,8 @@ static int main_nix_channel(int argc, char ** argv)
                 cmd = cList;
             } else if (*arg == "--update") {
                 cmd = cUpdate;
+            } else if (*arg == "--list-generations") {
+                cmd = cListGenerations;
             } else if (*arg == "--rollback") {
                 cmd = cRollback;
             } else {
@@ -237,6 +241,11 @@ static int main_nix_channel(int argc, char ** argv)
             case cUpdate:
                 update(StringSet(args.begin(), args.end()));
                 break;
+            case cListGenerations:
+                if (!args.empty())
+                    throw UsageError("'--list-generations' expects no arguments");
+                std::cout << runProgram(settings.nixBinDir + "/nix-env", false, {"--profile", profile, "--list-generations"}) << std::flush;
+                break;
             case cRollback:
                 if (args.size() > 1)
                     throw UsageError("'--rollback' has at most one argument");
