@@ -1,14 +1,15 @@
-#include "command.hh"
+#include "command-installable-value.hh"
 #include "shared.hh"
 #include "eval.hh"
 #include "attr-path.hh"
 #include "progress-bar.hh"
+#include "editor-for.hh"
 
 #include <unistd.h>
 
 using namespace nix;
 
-struct CmdEdit : InstallableCommand
+struct CmdEdit : InstallableValueCommand
 {
     std::string description() override
     {
@@ -24,23 +25,23 @@ struct CmdEdit : InstallableCommand
 
     Category category() override { return catSecondary; }
 
-    void run(ref<Store> store) override
+    void run(ref<Store> store, ref<InstallableValue> installable) override
     {
         auto state = getEvalState();
 
-        auto [v, pos] = installable->toValue(*state);
+        const auto [file, line] = [&] {
+            auto [v, pos] = installable->toValue(*state);
 
-        try {
-            pos = findPackageFilename(*state, *v, installable->what());
-        } catch (NoPositionInfo &) {
-        }
-
-        if (pos == noPos)
-            throw Error("cannot find position information for '%s", installable->what());
+            try {
+                return findPackageFilename(*state, *v, installable->what());
+            } catch (NoPositionInfo &) {
+                throw Error("cannot find position information for '%s", installable->what());
+            }
+        }();
 
         stopProgressBar();
 
-        auto args = editorFor(pos);
+        auto args = editorFor(file, line);
 
         restoreProcessContext();
 
