@@ -32,7 +32,7 @@ struct CmdStoreDelete : StorePathsCommand
           ;
     }
 
-    void run(ref<Store> store, std::vector<StorePath> && storePaths) override
+    void run(ref<Store> store, StorePaths && storePaths) override
     {
         auto & gcStore = require<GcStore>(*store);
 
