@@ -25,6 +25,18 @@ EOF
 
 (! nix flake check $flakeDir)
 
+cat > $flakeDir/flake.nix <<EOF
+{
+  outputs = { self, ... }: {
+    overlays.x86_64-linux.foo = final: prev: {
+    };
+  };
+}
+EOF
+
+checkRes=$(nix flake check $flakeDir 2>&1 && fail "nix flake check --all-systems should have failed" || true)
+echo "$checkRes" | grepQuiet "error: overlay is not a function, but a set instead"
+
 cat > $flakeDir/flake.nix <<EOF
 {
   outputs = { self }: {
