@@ -1,12 +1,14 @@
 source common.sh
 
+needLocalStore "uses some low-level store manipulations that aren’t available through the daemon"
+
 clearStore
 
 max=500
 
-reference=$NIX_STORE_DIR/aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
+reference=$NIX_STORE_DIR/aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa-bla
 touch $reference
-(echo $reference && echo && echo 0) | nix-store --register-validity 
+(echo $reference && echo && echo 0) | nix-store --register-validity
 
 echo "making registration..."
 
